@@ -64,18 +64,26 @@
 
 #include "log.h"
 
+typedef void (*osmo_e1dp_event_cb_t)(enum osmo_e1dp_msg_type, uint8_t, uint8_t, uint8_t, uint8_t *, int);
 
 /*! Internal representation of client program connected to the CTL socket */
 struct osmo_e1dp_client {
 	void *ctx;		/*!< talloc context */
 	struct osmo_fd ctl_fd;	/*!< osmo-fd wrapped unix domain (CTL) socket to @osmo-e1d@ */
+	osmo_e1dp_event_cb_t event_cb;
+				/*!< callback function for incoming events */
 };
 
 
 static int
 _e1dp_client_event(struct osmo_e1dp_client *clnt, struct msgb *msgb)
 {
-	/* FIXME */
+	struct osmo_e1dp_msg_hdr *hdr = msgb_l1(msgb);
+
+	if (!clnt->event_cb)
+		return -EINVAL;
+
+	clnt->event_cb(hdr->type, hdr->intf, hdr->line, hdr->ts, msgb_l2(msgb), msgb_l2len(msgb));
 	return 0;
 }
 
@@ -182,10 +190,13 @@ _e1dp_client_query_base(struct osmo_e1dp_client *clnt,
 	}
 
 	rc = osmo_e1dp_send(&clnt->ctl_fd, msgb, -1);
-	if (rc < 0)
+	if (rc < 0) {
+		msgb_free(msgb);
 		return rc;
+	}
 
 	msgb_free(msgb);
+	msgb = NULL;
 
 	/* Response */
 	int flags = fcntl(clnt->ctl_fd.fd, F_GETFL, 0);
@@ -378,8 +389,43 @@ osmo_e1dp_client_line_config(struct osmo_e1dp_client *clnt,
 	if (rc)
 		return rc;
 
-	if (msgb_l2len(msgb) != sizeof(struct osmo_e1dp_line_info))
+	if (msgb_l2len(msgb) != sizeof(struct osmo_e1dp_line_info)) {
+		msgb_free(msgb);
 		return -EPIPE;
+	}
+
+	msgb_free(msgb);
+
+	return 0;
+}
+
+/*! Set Sa-bits of a specific E1 line in osmo-e1d.
+ *  \param[in] clnt Client previously returned from osmo_e1dp_client_create().
+ *  \param[in] intf E1 interface number to configure.
+ *  \param[in] line E1 line number (within interface) to configure.
+ *  \param[in] sa_bits Sa bits to set on line.
+ *  \returns zero in case of success; negative in case of error. */
+int
+osmo_e1dp_client_set_sa_bits(struct osmo_e1dp_client *clnt, uint8_t intf, uint8_t line, uint8_t sa_bits)
+{
+	struct msgb *msgb;
+	struct osmo_e1dp_msg_hdr hdr;
+	int rc;
+
+	memset(&hdr, 0x00, sizeof(struct osmo_e1dp_msg_hdr));
+	hdr.type = E1DP_CMD_SABITS;
+	hdr.intf = intf;
+	hdr.line = line;
+	hdr.ts = E1DP_INVALID;
+
+	rc = _e1dp_client_query_base(clnt, &hdr, &sa_bits, 1, &msgb, NULL);
+	if (rc)
+		return rc;
+
+	if (msgb_l2len(msgb) != 0) {
+		msgb_free(msgb);
+		return -EPIPE;
+	}
 
 	msgb_free(msgb);
 
@@ -413,8 +459,10 @@ _client_ts_open(struct osmo_e1dp_client *clnt,
 	if (rc)
 		return rc;
 
-	if ((tsfd < 0) || (msgb_l2len(msgb) != sizeof(struct osmo_e1dp_ts_info)))
+	if ((tsfd < 0) || (msgb_l2len(msgb) != sizeof(struct osmo_e1dp_ts_info))) {
+		msgb_free(msgb);
 		return -EPIPE;
+	}
 
 	msgb_free(msgb);
 
@@ -455,3 +503,12 @@ osmo_e1dp_client_ts_open_force(struct osmo_e1dp_client *clnt,
 {
 	return _client_ts_open(clnt, intf, line, ts, mode, read_bufsize, E1DP_TS_OPEN_F_FORCE);
 }
+
+/*! Register event handler for incoming event messages.
+ *  \param[in] clnt Client previously returned from osmo_e1dp_client_create(). */
+void
+osmo_e1dp_client_event_register(struct osmo_e1dp_client *clnt,
+	void (*cb)(enum osmo_e1dp_msg_type event, uint8_t intf, uint8_t line, uint8_t ts, uint8_t *data, int len))
+{
+	clnt->event_cb = cb;
+}
