@@ -33,6 +33,8 @@
 #include <osmocom/core/utils.h>
 #include <osmocom/core/bit32gen.h>
 #include <osmocom/usb/libusb.h>
+#include <osmocom/e1d/proto.h>
+#include <osmocom/e1d/proto_srv.h>
 
 #include <libusb.h>
 
@@ -106,7 +108,8 @@ struct e1_usb_flow_entry {
 	struct libusb_transfer *xfr;
 };
 
-typedef int (*xfer_cb_t)(struct e1_usb_flow *flow, uint8_t *buf, int len);
+/* len == actual length, size == underlying size of buffer */
+typedef int (*xfer_cb_t)(struct e1_usb_flow *flow, uint8_t *buf, int len, size_t size);
 
 struct e1_usb_flow {
 	struct e1_line *line;
@@ -126,15 +129,42 @@ struct e1_usb_flow {
 // ---------------------------------------------------------------------------
 
 static int
-e1_usb_xfer_in(struct e1_usb_flow *flow, uint8_t *buf, int len)
+e1_usb_xfer_in(struct e1_usb_flow *flow, uint8_t *buf, int len, size_t size)
 {
 	if (len == 0)
 		return 0;
+
+	if (len < 4) {
+		LOGPLI(flow->line, DE1D, LOGL_ERROR, "IN EP %02x ISO packet truncated: len = %u\n",
+		       flow->ep, len);
+		line_ctr_add(flow->line, LINE_CTR_USB_ISO_TRUNC, 1);
+		return 0;
+	}
+
+	if (len > 4 && (len - 4) % 32) {
+		/* some ISO IN packet was truncated. Apparently this
+		 * does happen, see https://osmocom.org/issues/5490 -
+		 * there is little we can do here, but instead of the
+		 * earlier ASSERT, we just feed some garbage for the
+		 * last few timeslots, resulting in bit errors etc. */
+		LOGPLI(flow->line, DE1D, LOGL_ERROR, "IN EP %02x ISO packet truncated: len-4 = %u\n",
+		       flow->ep, len - 4);
+		line_ctr_add(flow->line, LINE_CTR_USB_ISO_TRUNC, 1);
+
+		/* The assumption here is that only the last E1 frame is
+		 * truncated, as we have no idea how many E1 frames the
+		 * USB device firmware wanted to send us. */
+		len += 32 - (len % 32);
+		/* don't overflow the underlying buffer */
+		if (len > (int) size)
+			len = size;
+	}
+
 	return e1_line_demux_in(flow->line, buf + 4, len - 4, buf[3] & 0xf);
 }
 
 static int
-e1_usb_xfer_out(struct e1_usb_flow *flow, uint8_t *buf, int len)
+e1_usb_xfer_out(struct e1_usb_flow *flow, uint8_t *buf, int len, size_t size)
 {
 	struct e1_line *line = flow->line;
 	struct e1_usb_line_data *ld = (struct e1_usb_line_data *) line->drv_data;
@@ -163,7 +193,7 @@ e1_usb_xfer_out(struct e1_usb_flow *flow, uint8_t *buf, int len)
 }
 
 static int
-e1_usb_xfer_fb(struct e1_usb_flow *flow, uint8_t *buf, int len)
+e1_usb_xfer_fb(struct e1_usb_flow *flow, uint8_t *buf, int len, size_t size)
 {
 	struct e1_usb_line_data *ld = (struct e1_usb_line_data *) flow->line->drv_data;
 
@@ -218,7 +248,8 @@ _e1uf_xfr(struct libusb_transfer *xfr)
 			}
 			flow->cb(flow,
 				libusb_get_iso_packet_buffer_simple(xfr, j),
-				(iso_pd->status == LIBUSB_TRANSFER_COMPLETED) ?  (int)iso_pd->actual_length : -1
+				(iso_pd->status == LIBUSB_TRANSFER_COMPLETED) ?  (int)iso_pd->actual_length : -1,
+				iso_pd->length
 			);
 			len += (iso_pd->length = flow->size);
 		}
@@ -229,7 +260,7 @@ _e1uf_xfr(struct libusb_transfer *xfr)
 				LOGPLI(flow->line, DE1D, LOGL_ERROR, "OUT EP %02x ISO packet %d failed with status %s\n",
 					flow->ep, j, get_value_string(libusb_status_str, iso_pd->status));
 			}
-			len += (iso_pd->length = flow->cb(flow, &xfr->buffer[len], flow->size));
+			len += (iso_pd->length = flow->cb(flow, &xfr->buffer[len], flow->size, flow->size));
 		}
 	}
 
@@ -302,7 +333,7 @@ e1uf_start(struct e1_usb_flow *flow)
 				len += (xfr->iso_packet_desc[j].length = flow->size);
 		} else {
 			for (j = 0; j < flow->ppx; j++)
-				len += (xfr->iso_packet_desc[j].length = flow->cb(flow, &flow->entries[i].buf[len], flow->size));
+				len += (xfr->iso_packet_desc[j].length = flow->cb(flow, &flow->entries[i].buf[len], flow->size, flow->size));
 		}
 
 		libusb_fill_iso_transfer(xfr, id->devh, flow->ep,
@@ -367,13 +398,38 @@ static void rx_interrupt_errcnt(struct e1_line *line, const struct ice1usb_irq_e
 	if ((errcnt->flags & ICE1USB_ERR_F_ALIGN_ERR) != (last->flags & ICE1USB_ERR_F_ALIGN_ERR)) {
 		LOGPLI(line, DE1D, LOGL_ERROR, "ALIGNMENT %s\n",
 			errcnt->flags & ICE1USB_ERR_F_ALIGN_ERR ? "LOST" : "REGAINED");
+		if ((errcnt->flags & ICE1USB_ERR_F_ALIGN_ERR)) {
+			osmo_e1dp_server_event(line->intf->e1d->srv, E1DP_EVT_LOF_ON,
+					       line->intf->id, line->id, 0, NULL, 0);
+		} else {
+			osmo_e1dp_server_event(line->intf->e1d->srv, E1DP_EVT_LOF_OFF,
+					       line->intf->id, line->id, 0, NULL, 0);
+		}
 	}
 
 	if ((errcnt->flags & ICE1USB_ERR_F_LOS) != (last->flags & ICE1USB_ERR_F_LOS)) {
 		LOGPLI(line, DE1D, LOGL_ERROR, "Rx Clock %s\n",
 			errcnt->flags & ICE1USB_ERR_F_LOS ? "LOST" : "REGAINED");
-		if (errcnt->flags & ICE1USB_ERR_F_LOS)
+		if (errcnt->flags & ICE1USB_ERR_F_LOS) {
 			line_ctr_add(line, LINE_CTR_LOS, 1);
+			osmo_e1dp_server_event(line->intf->e1d->srv, E1DP_EVT_LOS_ON,
+					       line->intf->id, line->id, 0, NULL, 0);
+		} else {
+			osmo_e1dp_server_event(line->intf->e1d->srv, E1DP_EVT_LOS_OFF,
+					       line->intf->id, line->id, 0, NULL, 0);
+		}
+	}
+
+	if ((errcnt->flags & ICE1USB_ERR_F_AIS) != (last->flags & ICE1USB_ERR_F_AIS)) {
+		LOGPLI(line, DE1D, LOGL_ERROR, "AIS (BLUE) %s\n",
+			errcnt->flags & ICE1USB_ERR_F_AIS ? "PRESENT" : "ABSENT");
+		if (errcnt->flags & ICE1USB_ERR_F_AIS) {
+			osmo_e1dp_server_event(line->intf->e1d->srv, E1DP_EVT_AIS_ON,
+					       line->intf->id, line->id, 0, NULL, 0);
+		} else {
+			osmo_e1dp_server_event(line->intf->e1d->srv, E1DP_EVT_AIS_OFF,
+					       line->intf->id, line->id, 0, NULL, 0);
+		}
 	}
 
 	if ((errcnt->flags & ICE1USB_ERR_F_RAI) != (last->flags & ICE1USB_ERR_F_RAI)) {
@@ -998,14 +1054,29 @@ _e1_usb_open_device(struct e1_daemon *e1d, struct libusb_device *dev, bool is_tr
 			goto next_interface;
 		}
 
-		/* Create data flows and start the line */
+		/* Configure, create data flows and start the line */
 
 		/* all supported devices have an IN endpoint */
+		e1_usb_ctrl_set_rx_cfg(line,
+			(line->usb.framing.rx == E1_FRAMING_MODE_CRC4) ?
+				ICE1USB_RX_MODE_MULTIFRAME :
+				ICE1USB_RX_MODE_FRAME
+		);
+
 		line_data->flow_in  = e1uf_create(line, e1_usb_xfer_in,  line_data->ep_in,  4, line_data->pkt_size, 4);
 		e1uf_start(line_data->flow_in);
 
 		/* e1-tracer has no OUT or FEEDBACK endpoint */
 		if (!is_tracer) {
+			e1_usb_ctrl_set_tx_cfg(line,
+				(line->usb.framing.tx == E1_FRAMING_MODE_CRC4) ?
+					ICE1USB_TX_MODE_TS0_CRC4_E :
+					ICE1USB_TX_MODE_TS0,
+				ICE1USB_TX_TIME_SRC_LOCAL,
+				ICE1USB_TX_EXT_LOOPBACK_OFF,
+				0x00
+			);
+
 			line_data->flow_out = e1uf_create(line, e1_usb_xfer_out, line_data->ep_out, 4, line_data->pkt_size, 4);
 			e1uf_start(line_data->flow_out);
 			line_data->flow_fb  = e1uf_create(line, e1_usb_xfer_fb,  line_data->ep_fb,  2, 3, 1);
